from os import listdir
from torch.nn import functional as F
import cv2
import torch
import numpy as np
import os
import random
import scipy.io as scio
import h5py


def build_datasets(root, dataset, size, n_select_bands, scale_ratio):
    # Imageh preprocessing, normalization for the pretrained resnet
    if dataset == 'Pavia':
        img = scio.loadmat(root + '/' + 'Pavia.mat')['pavia']*1.0
    elif dataset == 'PaviaU90':
        img = scio.loadmat(root + '/' + 'PaviaU90.mat')['paviaU90']*1.0
    elif dataset == 'Botswana':
        img = scio.loadmat(root + '/' + 'Botswana.mat')['Botswana']*1.0
    elif dataset == 'KSC':
        img = scio.loadmat(root + '/' + 'KSC.mat')['KSC']*1.0
    elif dataset == 'IndianP':
        img = scio.loadmat(root + '/' + 'Indian_pines_corrected.mat')['indian_pines_corrected']*1.0
    elif dataset == 'Washington':
        img = scio.loadmat(root + '/' + 'Washington_DC.mat')['img']*1.0
    elif dataset == 'Chikusei':
        img = scio.loadmat(root + '/' + 'Chikusei.mat')['Chikusei'] * 1.0
    elif dataset == 'Urban':
        img = scio.loadmat(root + '/' + 'Urban.mat')['Y']
        img = np.reshape(img, (162, 307, 307))*1.0
        img = np.swapaxes(img, 0,2)
    elif dataset == 'HHK':
        img = scio.loadmat(root + '/' + 'HHK.mat')['GF5_REF']*1.0
        img_msi = scio.loadmat(root + '/' + 'HHK.mat')['MSI'] * 1.0
    elif dataset == 'PYH':
        img = scio.loadmat(root + '/' + 'PYH.mat')['GF5_REF']*1.0
        img_msi = scio.loadmat(root + '/' + 'PYH.mat')['MSI'] * 1.0

############  设置平移的位置#####################
    hr_hs = np.pad(img, ((0, 0), (1, 0), (0, 0)), 'constant',
                 constant_values=((0, 0), (0, 0), (0, 0)))  # 行(上下平移)、列（左右平移）、Z        hr_hs is referenced image
    hr_ls= np.pad(img, ((0, 0), (0, 1), (0, 0)), 'constant', constant_values=((0, 0), (0, 0), (0, 0)))     ####  MSI    公开数据集
    # hr_ls = np.pad(img_msi, ((0, 2), (0, 0), (0, 0)), 'constant', constant_values=((0, 0), (0, 0), (0, 0)))  ####  MSI   遥感数据集
######  hr_hs 预处理##############
    print (hr_hs.shape)
    max = np.max(hr_hs)
    min = np.min(hr_hs)
    hr_hs = 255*((hr_hs - min) / (max - min + 0.0))

    # throwing up the edge
    hr_hs_w_edge = hr_hs.shape[0]//scale_ratio*scale_ratio-hr_hs.shape[0]
    hr_hs_h_edge = hr_hs.shape[1]//scale_ratio*scale_ratio-hr_hs.shape[1]
    hr_hs_w_edge = -1  if hr_hs_w_edge==0  else  hr_hs_w_edge
    hr_hs_h_edge = -1  if hr_hs_h_edge==0  else  hr_hs_h_edge
    hr_hs = hr_hs[:hr_hs_w_edge, :hr_hs_h_edge, :]

    # cropping area
    width, height, n_bands = hr_hs.shape
    hr_hs_w_str = (width - size) // 2
    hr_hs_h_str = (height - size) // 2
    hr_hs_w_end = hr_hs_w_str + size
    hr_hs_h_end = hr_hs_h_str + size
    hr_hs_copy = hr_hs.copy()

    ######  hr_ls 预处理##############
    print(hr_ls.shape)
    max = np.max(hr_ls)
    min = np.min(hr_ls)
    hr_ls = 255 * ((hr_ls - min) / (max - min + 0.0))

    # throwing up the edge
    hr_ls_w_edge = hr_ls.shape[0] // scale_ratio * scale_ratio - hr_ls.shape[0]
    hr_ls_h_edge = hr_ls.shape[1] // scale_ratio * scale_ratio - hr_ls.shape[1]
    hr_ls_w_edge = -1 if hr_ls_w_edge == 0 else hr_ls_w_edge
    hr_ls_h_edge = -1 if hr_ls_h_edge == 0 else hr_ls_h_edge
    hr_ls = hr_ls[:hr_ls_w_edge, :hr_ls_h_edge, :]

    # cropping area
    # width, height, n_select_bands = hr_ls.shape           ########遥感数据集
    width, height, n_bands = hr_ls.shape                 ##########公开数据集

    hr_ls_w_str = (width - size) // 2
    hr_ls_h_str = (height - size) // 2
    hr_ls_w_end = hr_ls_w_str + size
    hr_ls_h_end = hr_ls_h_str + size
    hr_ls_copy = hr_ls.copy()
##############################################
    # test sample
    gap_bands = n_bands / (n_select_bands-1.0)
    hr_hs_test_ref = hr_hs_copy[hr_hs_w_str:hr_hs_w_end, hr_hs_h_str:hr_hs_h_end, :].copy()
    test_lr = cv2.GaussianBlur(hr_hs_test_ref, (5,5), 2)
    test_lr = cv2.resize(test_lr, (size//scale_ratio, size//scale_ratio))               ###### 低分辨率的HSI


    hr_ls_test_ref = hr_ls_copy[hr_ls_w_str:hr_ls_w_end, hr_ls_h_str:hr_ls_h_end, :].copy()

    # test_hr = hr_ls_test_ref                                                              ######### 遥感数据集

    test_hr = hr_ls_test_ref[:,:,0][:,:,np.newaxis]                                        ##########  公开数据集
    for i in range(1, n_select_bands-1):
        test_hr = np.concatenate((test_hr, hr_ls_test_ref[:,:,int(gap_bands*i)][:,:,np.newaxis],), axis=2)
    test_hr = np.concatenate((test_hr, hr_ls_test_ref[:,:,n_bands-1][:,:,np.newaxis],), axis=2)      ############高分辨率的MSI

    ##############################################
    # training sample
    hr_hs[hr_hs_w_str:hr_hs_w_end,hr_hs_h_str:hr_hs_h_end,:] = 0
    hr_hs_train_ref = hr_hs
    train_lr = cv2.GaussianBlur(hr_hs_train_ref, (5,5), 2)
    train_lr = cv2.resize(train_lr, (train_lr.shape[1]//scale_ratio, train_lr.shape[0]//scale_ratio))      ###### 低分辨率的HSI

    hr_ls[hr_ls_w_str:hr_ls_w_end, hr_ls_h_str:hr_ls_h_end, :] = 0
    hr_ls_train_ref = hr_ls

    # train_hr = hr_ls_train_ref                                               ######### 遥感数据集

    train_hr = hr_ls_train_ref[:,:,0][:,:,np.newaxis]                         ##########  公开数据集
    for i in range(1, n_select_bands-1):
        train_hr = np.concatenate((train_hr, hr_ls_train_ref[:,:,int(gap_bands*i)][:,:,np.newaxis],), axis=2)
    train_hr = np.concatenate((train_hr, hr_ls_train_ref[:,:,n_bands-1][:,:,np.newaxis],), axis=2)              ############高分辨率的MSI

######################################
    train_ref = torch.from_numpy(hr_hs_train_ref).permute(2,0,1).unsqueeze(dim=0).float()
    train_lr = torch.from_numpy(train_lr).permute(2,0,1).unsqueeze(dim=0).float()
    train_hr = torch.from_numpy(train_hr).permute(2,0,1).unsqueeze(dim=0).float()
    test_ref = torch.from_numpy(hr_hs_test_ref).permute(2,0,1).unsqueeze(dim=0).float()
    test_lr = torch.from_numpy(test_lr).permute(2,0,1).unsqueeze(dim=0).float()
    test_hr = torch.from_numpy(test_hr).permute(2,0,1).unsqueeze(dim=0).float()

    return [train_ref, train_lr, train_hr], [test_ref, test_lr, test_hr]